use crate::prelude::*;

use std::os::raw::c_void;
use std::ptr;

#[repr(C)]
pub struct VkRayTracingPipelineCreateInfoKHR {
    pub sType: VkStructureType,
    pub pNext: *const c_void,
    pub flags: VkPipelineCreateFlagBits,
    pub stageCount: u32,
    pub pStages: *const VkPipelineShaderStageCreateInfo,
    pub groupCount: u32,
    pub pGroups: *const VkRayTracingShaderGroupCreateInfoKHR,
    pub maxPipelineRayRecursionDepth: u32,
    pub pLibraryInfo: *const VkPipelineLibraryCreateInfoKHR,
    pub pLibraryInterface: *const VkRayTracingPipelineInterfaceCreateInfoKHR,
    pub pDynamicState: *const VkPipelineDynamicStateCreateInfo,
    pub layout: VkPipelineLayout,
    pub basePipelineHandle: VkPipeline,
    pub basePipelineIndex: i32,
}

impl VkRayTracingPipelineCreateInfoKHR {
    pub fn new(
        flags: impl Into<VkPipelineCreateFlagBits>,
        stages: &[VkPipelineShaderStageCreateInfo],
        groups: &[VkRayTracingShaderGroupCreateInfoKHR],
        max_pipeline_ray_recursion_depth: u32,
        libraries: &VkPipelineLibraryCreateInfoKHR,
        library_interface: &VkRayTracingPipelineInterfaceCreateInfoKHR,
        dynamic_state: &VkPipelineDynamicStateCreateInfo,
        layout: VkPipelineLayout,
    ) -> Self {
        VkRayTracingPipelineCreateInfoKHR {
            sType: VK_STRUCTURE_TYPE_RAY_TRACING_PIPELINE_CREATE_INFO_KHR,
            pNext: ptr::null(),
            flags: flags.into(),
            stageCount: stages.len() as u32,
            pStages: stages.as_ptr(),
            groupCount: groups.len() as u32,
            pGroups: groups.as_ptr(),
            maxPipelineRayRecursionDepth: max_pipeline_ray_recursion_depth,
            pLibraryInfo: libraries,
            pLibraryInterface: library_interface,
            pDynamicState: dynamic_state,
            layout,
            basePipelineHandle: VkPipeline::NULL_HANDLE,
            basePipelineIndex: -1,
        }
    }

    pub fn set_base_pipeline(&mut self, pipeline: VkPipeline, index: i32) {
        self.basePipelineHandle = pipeline;
        self.basePipelineIndex = index;
    }
}
